#include "QDebug.h"
#include "glwidget.h"

GLWidget::GLWidget(QWidget *parent) : QOpenGLWidget(parent) {
    red = green = blue = 0;
    xRot = yRot = zRot = 0;
}

GLWidget::~GLWidget() {
}

void GLWidget::getColor(GLfloat& _red, GLfloat& _green, GLfloat& _blue) {
    _red = red; _green = green; _blue = blue;
}

void GLWidget::setColor(GLfloat _red, GLfloat _green, GLfloat _blue) {
    if (_red != red || _green != green || _blue != blue) {
        red = _red; green = _green; blue = _blue;

        makeCurrent();
        GLfloat lightColor[4] = {red, green, blue, 1};
        glLightfv(GL_LIGHT0, GL_AMBIENT, lightColor);

        emit colorChanged(red, green, blue);
        update();
    }
}

void GLWidget::setXRot(GLfloat angle) {
    if (angle != xRot) {
        xRot = angle;
        emit xRotChanged(angle);
        update();
    }
}

void GLWidget::setYRot(GLfloat angle) {
    if (angle != yRot) {
        yRot = angle;
        emit yRotChanged(angle);
        update();
    }
}

void GLWidget::setZRot(GLfloat angle) {
    if (angle != zRot) {
        zRot = angle;
        emit zRotChanged(angle);
        update();
    }
}

void GLWidget::initializeGL() {
    initializeOpenGLFunctions();   // ne pas oublier !
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glShadeModel(GL_SMOOTH);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    GLfloat lightPosition[4] = {0, 0, 10, 1};
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
    GLfloat lightColor[4] = {red, green, blue, 1};
    glLightfv(GL_LIGHT0, GL_AMBIENT, lightColor);
}

void GLWidget::paintGL() {
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    glTranslatef(0.0, 0.0, -10.0);
    glRotatef(xRot, 1.0, 0.0, 0.0);
    glRotatef(yRot, 0.0, 1.0, 0.0);
    glRotatef(zRot, 0.0, 0.0, 1.0);
    draw();
}

void GLWidget::resizeGL(int w, int h) {
    int side = qMin(w, h);
    glViewport((w - side) / 2, (h - side) / 2, side, side);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
#ifdef QT_OPENGL_ES_1
    glOrthof(-2, +2, -2, +2, 1.0, 15.0);
#else
    glOrtho(-2, +2, -2, +2, 1.0, 15.0);
#endif
    glMatrixMode(GL_MODELVIEW);
}

void GLWidget::draw() {
    glBegin(GL_QUADS);
    glNormal3f(0,0,-1);
    glVertex3f(-1,-1,0);
    glVertex3f(-1,1,0);
    glVertex3f(1,1,0);
    glVertex3f(1,-1,0);
    glEnd();

    glBegin(GL_TRIANGLES);
    glNormal3f(0,-1,0.707);
    glColor3f(1.0f, 0.0f, 0.0f);
    glVertex3f(-1,-1,0);
    glColor3f(1.0f, 1.0f, 0.0f);
    glVertex3f(1,-1,0);
    glColor3f(1.0f, 0.0f, 1.0f);
    glVertex3f(0,0,1.2);
    glEnd();

    glBegin(GL_TRIANGLES);
    glNormal3f(1,0, 0.707);
    glVertex3f(1,-1,0);
    glVertex3f(1,1,0);
    glVertex3f(0,0,1.2);
    glEnd();

    glBegin(GL_TRIANGLES);
    glNormal3f(0,1,0.707);
    glVertex3f(1,1,0);
    glVertex3f(-1,1,0);
    glVertex3f(0,0,1.2);
    glEnd();

    glBegin(GL_TRIANGLES);
    glNormal3f(-1,0,0.707);
    glVertex3f(-1,1,0);
    glVertex3f(-1,-1,0);
    glVertex3f(0,0,1.2);
    glEnd();
}
